//Jack-in-the-Box
//INFO4320
//JEREMY E. BLUM

#include <Servo.h>                         //Include the Servo Library
#include "pitches.h"                       //Include the Tone Library
 
Servo myservo;                             // create servo object to control a servo 

const int doorInterrupt = 0;               // Interrupt for Door Trip Sensor  - Pin 2 = 0, Pin 3 = 1
const int doorPin       = 2;               // Pin number for door trip sensor
const int servoPin      = 6;               // Pin to control the Servo Motor
const int buttonPinA    = 7;               // Pin to first unlock button
const int buttonPinB    = 8;               // Pin to second unlock button
const int buttonPinC    = 9;               // Pin to third unlock button
const int speakerPin    = 10;              // Pin to Alarm Speaker

const int lidOpen       = 105;             //Servo Value to Open Lid
const int lidClosed     = 33;              //Servo Value to Close Lid

byte current_buttonA    = LOW;             //State of buttonA
byte old_buttonA        = LOW;             //Previous state of buttonA
byte current_buttonB    = LOW;             //State of buttonB
byte old_buttonB        = LOW;             //Previous state of buttonB
byte current_buttonC    = LOW;             //State of buttonC
byte old_buttonC        = LOW;             //Previous state of buttonC

const int len = 6;                         //DEFINE THE LENGTH OF THE COMBO ARRAY (and consequently the attempt array)
const int combo[len] = {1, 2, 3, 3, 2, 1}; //DEFINE THE COMBO ARRAY - The Button Press order to unlock the box (button A = 1, B = 2, C = 3)
int attempt[len];                          //Inialize the attempt array

// Pass Tones:
const int melody_pass[] = {NOTE_C4,NOTE_G3,NOTE_G3,NOTE_A3,NOTE_G3,0,NOTE_B3,NOTE_C4};
const int noteDurations_pass[] = {4,8,8,4,4,4,4,4 };

//Fail Tones:
const int melody_fail[] = {NOTE_D4,NOTE_D3,NOTE_D2,NOTE_D1};
const int noteDurations_fail[] = {4,4,4,1};

void setup() 
{ 
  myservo.attach(servoPin);             //Attaches the servo to the servo object 

  pinMode(doorPin,    INPUT);           //Door trip sensor is an input
  pinMode(servoPin,   OUTPUT);          //The servo is an output
  pinMode(buttonPinA, INPUT);           //The first button is an input
  pinMode(buttonPinB, INPUT);           //The seconds button is an input
  pinMode(buttonPinC, INPUT);           //The third button is an input
  pinMode(speakerPin, OUTPUT);          //The fourth Button is an input
  
  attachInterrupt(doorInterrupt, alarm, RISING);    //Attach the door interrupt

  
  //Initialize the attempt array with zeros
  for (int i = 0; i < len; i = i++) attempt[i] = 0;
  
  Serial.begin(9600);                   //Open Serial Connection for debugging

} 

void alarm()
{
  Serial.println ("ALARM!");
  tone(speakerPin, NOTE_G3, 1000);
  delayMicroseconds(1000000);
}
void bogus()
{
}

byte read_button(byte old_value, int pin)              //Debounced Button Reading function
{
  byte current_button = digitalRead(pin);              //Read the Current Button State
  if ((old_value == LOW) && (current_button == HIGH))  //If it is transitioning to high...
  {
    delay(5);                                          //Wait 5ms
    current_button = digitalRead(pin);                 //Then read it's "high" value
  }
  if ((old_value == HIGH) && (current_button ==LOW))   //If it is transitioning to low...
  {
    delay(5);                                          //Wait 5ms
    current_button=digitalRead(pin);                   //Then read the value
  }
  return(current_button);                              //return the de-bounced button value
} 
 
int  which_button()
{
  current_buttonA = read_button(old_buttonA, buttonPinA);
  current_buttonB = read_button(old_buttonB, buttonPinB);
  current_buttonC = read_button(old_buttonC, buttonPinC);
  if      ((old_buttonA == LOW) && (current_buttonA == HIGH)) return (1);
  else if ((old_buttonB == LOW) && (current_buttonB == HIGH)) return (2);
  else if ((old_buttonC == LOW) && (current_buttonC == HIGH)) return (3);
  else    return (0);
  old_buttonA = current_buttonA;
  old_buttonB = current_buttonB;
  old_buttonC = current_buttonC;
}

void play_song(boolean pass)
{
  if (pass)
  {
    for (int thisNote = 0; thisNote < 8; thisNote++)
     {
       int noteDuration_pass = 1000/noteDurations_pass[thisNote];
       tone(speakerPin, melody_pass[thisNote],noteDuration_pass);
       int pauseBetweenNotes = noteDuration_pass * 1.30;
       delay(pauseBetweenNotes);
     }
  }
  else
  {
     for (int thisNote = 0; thisNote < 8; thisNote++)
     {
       int noteDuration_fail = 1000/noteDurations_fail[thisNote];
       tone(speakerPin, melody_fail[thisNote],noteDuration_fail);
       int pauseBetweenNotes = noteDuration_fail * 1.30;
       delay(pauseBetweenNotes);
     }
  }
}
 
void loop() 
{ 
  
 myservo.write(lidClosed);                    //The lid should always start closed
 delay(15);                                   //Give the motor time to actuate
 
 //This will read the button press attempts!
 for (int i=0; i<len; i++)
 {
   while (which_button() == 0);                //Stalls while waiting for a button to be pressed
   attempt[i] = which_button();               //Adds button press to tracking array
   
   for (int i = 0; i < len; i = i++) Serial.print(attempt[i]);
   
   while (which_button() == attempt[i]);         //Wait for the button to be released 
   Serial.println("");
 }
 
 //Was the correct Code Entered?
 int correct = 0;
 for (int i=0; i<len; i++)
 {
   if (attempt[i] == combo[i]) correct++;  
 }
 boolean good = false;
 if (correct == len) good = true;
 
 //If the Correct Code was entered...
 if (good)
 {
   detachInterrupt(doorInterrupt);    //Disable Alarm
   myservo.write(lidOpen);            //Open the lid
   Serial.println("PASS!");      
   play_song(true);                   //Play a tune!
   
   //Keep lid open until all three buttons are pressed at once
   while (digitalRead(buttonPinA) == LOW || digitalRead(buttonPinB) == LOW || digitalRead(buttonPinC) == LOW);
   myservo.write(lidClosed);
   delay (2000);     //Give time for buttons to be released
   attachInterrupt(doorInterrupt, bogus, RISING);    //Attach the door interrupt
   attachInterrupt(doorInterrupt, alarm, RISING);
 }
 //If the wrong code was entered...
 else
 {
   Serial.println("FAIL!");
   play_song(false);                  //Play FAIL Tune

 }
 
 for (int i = 0; i < len; i = i++) attempt[i] = 0;  //Reset Attempt array to zeros   
} 
